local assets =
{
    Asset("ANIM", "anim/greenbackpack.zip"),
    Asset("ANIM", "anim/swap_greenbackpack.zip"),

    Asset("ATLAS", "images/inventoryimages/greenbackpack.xml"),
    Asset("IMAGE", "images/inventoryimages/greenbackpack.tex")
}

local function OnBlocked(owner) 
    owner.SoundEmitter:PlaySound("dontstarve/wilson/hit_armour") 
end

local function onequip(inst, owner) 
    owner.AnimState:OverrideSymbol("swap_body", "swap_greenbackpack", "greenbackpack")
    owner.AnimState:OverrideSymbol("swap_body", "swap_greenbackpack", "swap_body")
    if inst.components.container ~= nil then
        inst.components.container:Open(owner)
    end
end

local function onunequip(inst, owner) 
    owner.AnimState:ClearOverrideSymbol("swap_body")
    owner.AnimState:ClearOverrideSymbol("greenbackpack")
    inst:RemoveEventCallback("blocked", OnBlocked, owner)
    if inst.components.container ~= nil then
    inst.components.container:Close(owner)
    end
end

local function onopen(inst)
	inst.SoundEmitter:PlaySound("dontstarve/wilson/backpack_open", "open")
end

local function onclose(inst)
	inst.SoundEmitter:PlaySound("dontstarve/wilson/backpack_close", "open")
end

local function fn(Sim)
	local inst = CreateEntity()
    
    inst.entity:AddTransform()
    inst.entity:AddAnimState()
    inst.entity:AddSoundEmitter()
    inst.entity:AddMiniMapEntity()
    inst.entity:AddNetwork()
    
    inst.AnimState:SetBank("backpack1")
    inst.AnimState:SetBuild("swap_greenbackpack")
    inst.AnimState:PlayAnimation("anim")

    MakeInventoryPhysics(inst)

    inst:AddTag("backpack")
    inst:AddTag("irreplaceable")

	local minimap = inst.entity:AddMiniMapEntity()
	minimap:SetIcon("greenbackpack.png")

    inst.foleysound = "dontstarve/movement/foley/backpack"

    if not TheWorld.ismastersim then
        inst:DoTaskInTime(0.314, function() 
            inst.replica.container:WidgetSetup("piggyback")
            end)

        return inst
    end

    inst.entity:SetPristine()

    inst:AddComponent("characterspecific")

    inst:AddComponent("inspectable")

    inst:AddComponent("armor")
    inst.components.armor:InitCondition(TUNING.ARMORMARBLE * 1000000000000000000, TUNING.ARMORWOOD_ABSORPTION)

    inst:AddComponent("insulator")
    inst.components.insulator:SetInsulation(TUNING.INSULATION_TINY)
 
    inst:AddComponent("container")
    inst.components.container:WidgetSetup("piggyback")

    inst:AddComponent("inventoryitem")
    inst.components.inventoryitem.keepondeath = true
    inst.components.inventoryitem.imagename = "greenbackpack"
    inst.components.inventoryitem.atlasname = "images/inventoryimages/greenbackpack.xml"

    inst:AddComponent("inspectable")
    
    inst.components.container.onopenfn = onopen
    inst.components.container.onclosefn = onclose
    
    inst:AddComponent("equippable")
    inst.components.equippable.keepondeath = true
	inst.components.equippable.equipslot = EQUIPSLOTS.BODY
	
	if EQUIPSLOTS["PACK"] then
		inst.components.equippable.equipslot = EQUIPSLOTS.PACK
	else
		inst.components.equippable.equipslot = EQUIPSLOTS.BODY
	end
    inst.components.equippable:SetOnEquip(onequip)
    inst.components.equippable:SetOnUnequip(onunequip)
    inst.components.equippable.walkspeedmult = 1.0

    MakeHauntableLaunchAndDropFirstItem(inst)
    
    return inst
end

return Prefab( "common/inventory/greenbackpack", fn, assets) 